@@ -1372,13 +1372,27 @@ preview_update (Preview *p)
   else
     p->aspect = width/height;
 
-  max_width  = 0.5*gdk_screen_width ();
-  max_height = 0.5*gdk_screen_height ();
+  if (surface_changed)
+    {
+      max_width  = 0.5 * gdk_screen_width();
+      max_height = 0.5 * gdk_screen_height();
+    }
+  else
+    {
+      max_width  = p->window->allocation.width;
+      max_height = p->window->allocation.height;
+    }
 
   if (p->surface_unit != SANE_UNIT_PIXEL)
     {
       width  = max_width;
-      height = max_height;
+      height = width / p->aspect;
+
+      if (height > max_height)
+	{
+	  height = max_height;
+	  width  = height * p->aspect;
+	}
     }
   else
     {
