@@ -968,14 +968,14 @@ main (int argc, char **argv)
 			     build_preferences_menu (dialog));
   gtk_widget_show (menu_bar_item);
 
-  gtk_container_add (GTK_CONTAINER (vbox), menu_bar);
+  gtk_box_pack_start (GTK_BOX (vbox), menu_bar, FALSE, FALSE, 0);
   gtk_widget_show (menu_bar);
 
   /* add device info at top: */
   frame = gtk_frame_new (0);
   gtk_container_border_width (GTK_CONTAINER (frame), 8);
   gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_ETCHED_IN);
-  gtk_container_add (GTK_CONTAINER (vbox), frame);
+  gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE, 0);
   gtk_widget_show (frame);
 
   win.device_info_label = gtk_label_new ("");
@@ -1067,13 +1067,11 @@ main (int argc, char **argv)
 			  win.params.pixels_per_line, win.params.lines);
 
   /* buttons at the bottom: */
-  button = gtk_button_new ();
+  button = gtk_button_new_with_label ("Play");
+  win.play_stop_label = GTK_BIN (button)->child;
   gtk_signal_connect (GTK_OBJECT (button), "clicked",
 		      (GtkSignalFunc) play_stop_button, dialog);
-  win.play_stop_label = gtk_label_new ("Play");
-  gtk_container_add (GTK_CONTAINER (button), win.play_stop_label);
-  gtk_container_add (GTK_CONTAINER (vbox), button);
-  gtk_widget_show (win.play_stop_label);
+  gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);
   gtk_widget_show (button);
 
   gtk_widget_show (win.shell);
